const ITrader = require('./ITrader')
const CCXTExchange = require('../exchange/CCXTExchange')
global.JsmexSymbol = global.JsmexSymbol || require('../utils/Symbol')
const _ = require('../../static_dependencies/lodash/lodash.js')

/**
* CCXTTrader class
*
* @author jsb4ch <jsb4ch@jsmex.io>
* @since 1.0.0
*/
class CCXTTrader extends ITrader {
  constructor (options) {
    super(options)
    this.ccxtExchanges = {}
    this.defaultExchange = null
    for (const exchangeName in (typeof this.options.exchanges === 'object' ? this.options.exchanges : ['jsmex'])) {
      const exchangeoptions = _.merge({ exchangeName }, ...[this.options.exchanges[exchangeName] || {}, options || {}, this._apiToken ? { apiToken: this._apiToken } : {}])
      if (!this.defaultExchange && (exchangeoptions.isMaster || exchangeoptions.default)) {
        this.defaultExchange = exchangeName
      }
      this.ccxtExchanges[exchangeName] = new CCXTExchange(exchangeoptions)
      if (!this.ccxtExchanges[exchangeName]) {
        throw new Error('Invalid ccxt exchange declaration')
      }
    }
    if (!this.defaultExchange) {
      this.defaultExchange = Object.keys(this.ccxtExchanges)[0] || null
    }
  }

  /**
    * CCXTTrader switch method
    * switch default exchange name, throw error if the exchange name is invalid
    *
    * @author jsb4ch <jsb4ch@jsmex.io>
    * @param {String} exchangeName
    * @returns {Boolean}
    * @since 1.0.0
    */
  switch (exchangeName) {
    if (Object.keys(this.ccxtExchanges).includes(exchangeName)) {
      this.defaultExchange = exchangeName
      return true
    } else {
      throw new Error('Invalid exchange name')
    }
  }

  ccxtFromArgs (params, ...args) {
    if (global.JsmexSymbol.isValid(params.symbol)) {
      const exchangeName = global.JsmexSymbol.exchangeName(params.symbol)
      return exchangeName && this.ccxtExchanges[exchangeName]
        ? this.ccxtExchanges[exchangeName]
        : this.ccxtExchanges[this.defaultExchange]
    } else {
      return this.ccxtExchanges[this.defaultExchange]
    }
  }

  fetchDepth (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).fetchDepth, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('fetchDepth request')
    return ccxtExchange.fetchDepth(ccxtParams)
  }

  fetchOHLCV (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).fetchOHLCV, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('fetchOHLCV request')
    return ccxtExchange.fetchOHLCV(ccxtParams)
  }

  loadMarkets (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).loadMarkets, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('loadMarkets')
    return ccxtExchange.loadMarkets(ccxtParams)
  }

  cancelOrder (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).cancelOrder, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('cancelOrder', ccxtParams.symbol, ccxtParams.id)
    return ccxtExchange.cancelOrder(ccxtParams)
  }

  cancelAllOrders (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).cancelAllOrders, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('cancelAllOrders')
    return ccxtExchange.cancelAllOrders(ccxtParams)
  }

  fetchOrders (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).fetchOrders, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('fetchOrders request')
    return ccxtExchange.fetchOrders(ccxtParams)
  }

  fetchTransactions (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).fetchTransactions, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('fetchTransactions request')
    return ccxtExchange.fetchTransactions(ccxtParams)
  }

  putOrder (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).putOrder, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('createOrder', ccxtParams.symbol, ccxtParams.type, ccxtParams.side, ccxtParams.price, ccxtParams.amount)
    return ccxtExchange.createOrder(ccxtParams)
  }

  pipeline (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).pipeline, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('pipeline request')
    return ccxtExchange.pipeline(ccxtParams)
  }

  fetchBalance (params) {
    let ccxtParams = {}
    _.merge(ccxtParams, ...[(this.options.ccxt || {}).balance, { symbol: this.options.defaultSymbol }, (params || {})])
    const ccxtExchange = this.ccxtFromArgs(ccxtParams)
    global.log('balance request')
    return ccxtExchange.balance(ccxtParams)
  }
}

module.exports = CCXTTrader
